@@ -35,6 +35,6 @@
 
 #define NVC0_3D_MACRO_QUERY_BUFFER_WRITE			0x00003858
 
-#define NVC0_COMPUTE_MACRO_LAUNCH_GRID_INDIRECT       0x00003860
+#define NVC0_CP_MACRO_LAUNCH_GRID_INDIRECT       0x00003860
 
 #endif /* __NVC0_MACROS_H__ */
