@@ -63,9 +63,9 @@ NineVertexBuffer9_dtor( struct NineVertexBuffer9 *This )
 }
 
 struct pipe_resource *
-NineVertexBuffer9_GetResource( struct NineVertexBuffer9 *This )
+NineVertexBuffer9_GetResource( struct NineVertexBuffer9 *This, unsigned *offset )
 {
-    return NineBuffer9_GetResource(&This->base);
+    return NineBuffer9_GetResource(&This->base, offset);
 }
 
 HRESULT NINE_WINAPI
