@@ -50,6 +50,7 @@
 #define VL_VA_PSCREEN(ctx) (VL_VA_DRIVER(ctx)->vscreen->pscreen)
 
 #define VL_VA_MAX_IMAGE_FORMATS 9
+#define VL_VA_ENC_GOP_COEFF 16
 
 static inline enum pipe_video_chroma_format
 ChromaToPipe(int format)
@@ -219,6 +220,20 @@ typedef struct {
    struct pipe_sampler_view *sampler;
 } vlVaSubpicture;
 
+typedef struct {
+   VABufferType type;
+   unsigned int size;
+   unsigned int num_elements;
+   void *data;
+   struct {
+      struct pipe_resource *resource;
+      struct pipe_transfer *transfer;
+   } derived_surface;
+   unsigned int export_refcount;
+   VABufferInfo export_state;
+   unsigned int coded_size;
+} vlVaBuffer;
+
 typedef struct {
    struct pipe_video_codec templat, *decoder;
    struct pipe_video_buffer *target;
@@ -242,8 +257,10 @@ typedef struct {
    } mpeg4;
 
    struct vl_deint_filter *deint;
-   struct vlVaBuffer *coded_buf;
+   vlVaBuffer *coded_buf;
    int target_id;
+   bool first_single_submitted;
+   int gop_coeff;
    bool needs_begin_frame;
 } vlVaContext;
 
@@ -254,20 +271,6 @@ typedef struct {
    unsigned int rt_format;
 } vlVaConfig;
 
-typedef struct {
-   VABufferType type;
-   unsigned int size;
-   unsigned int num_elements;
-   void *data;
-   struct {
-      struct pipe_resource *resource;
-      struct pipe_transfer *transfer;
-   } derived_surface;
-   unsigned int export_refcount;
-   VABufferInfo export_state;
-   unsigned int coded_size;
-} vlVaBuffer;
-
 typedef struct {
    struct pipe_video_buffer templat, *buffer;
    struct util_dynarray subpics; /* vlVaSubpicture */
@@ -275,6 +278,7 @@ typedef struct {
    vlVaBuffer *coded_buf;
    void *feedback;
    unsigned int frame_num_cnt;
+   bool force_flushed;
 } vlVaSurface;
 
 // Public functions:
