@@ -33,17 +33,19 @@
 
 #include "dri_util.h"
 #include "intel_bufmgr.h"
-#include "brw_device_info.h"
+#include "common/gen_device_info.h"
 #include "i915_drm.h"
 #include "xmlconfig.h"
 
 struct intel_screen
 {
    int deviceID;
-   const struct brw_device_info *devinfo;
+   struct gen_device_info devinfo;
 
    __DRIscreen *driScrnPriv;
 
+   uint64_t max_gtt_map_object_size;
+
    bool no_hw;
 
    bool hw_has_swizzling;
@@ -55,17 +57,22 @@ struct intel_screen
     */
    bool has_resource_streamer;
 
-   /**
-    * Does the current hardware and kernel support MI_MATH and
-    * MI_LOAD_REGISTER_REG?
-    */
-   bool has_mi_math_and_lrr;
-
    /**
     * Does the kernel support context reset notifications?
     */
    bool has_context_reset_notification;
 
+   /**
+    * Does the kernel support features such as pipelined register access to
+    * specific registers?
+    */
+   unsigned kernel_features;
+#define KERNEL_ALLOWS_SOL_OFFSET_WRITES             (1<<0)
+#define KERNEL_ALLOWS_PREDICATE_WRITES              (1<<1)
+#define KERNEL_ALLOWS_MI_MATH_AND_LRR               (1<<2)
+#define KERNEL_ALLOWS_HSW_SCRATCH1_AND_ROW_CHICKEN3 (1<<3)
+#define KERNEL_ALLOWS_COMPUTE_DISPATCH              (1<<4)
+
    dri_bufmgr *bufmgr;
 
    /**
@@ -117,4 +124,34 @@ void aub_dump_bmp(struct gl_context *ctx);
 const int*
 intel_supported_msaa_modes(const struct intel_screen  *screen);
 
+static inline bool
+can_do_pipelined_register_writes(const struct intel_screen *screen)
+{
+   return screen->kernel_features & KERNEL_ALLOWS_SOL_OFFSET_WRITES;
+}
+
+static inline bool
+can_do_hsw_l3_atomics(const struct intel_screen *screen)
+{
+   return screen->kernel_features & KERNEL_ALLOWS_HSW_SCRATCH1_AND_ROW_CHICKEN3;
+}
+
+static inline bool
+can_do_mi_math_and_lrr(const struct intel_screen *screen)
+{
+   return screen->kernel_features & KERNEL_ALLOWS_MI_MATH_AND_LRR;
+}
+
+static inline bool
+can_do_compute_dispatch(const struct intel_screen *screen)
+{
+   return screen->kernel_features & KERNEL_ALLOWS_COMPUTE_DISPATCH;
+}
+
+static inline bool
+can_do_predicate_writes(const struct intel_screen *screen)
+{
+   return screen->kernel_features & KERNEL_ALLOWS_PREDICATE_WRITES;
+}
+
 #endif
